##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
Python Product for XML Documents
"""

import Node
from Parser import Parser, ParseError
from Globals import HTMLFile, MessageDialog, Persistent
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from AccessControl.Role import RoleManager
from DateTime import DateTime
import string
from urllib import quote

class Manageable(PropertyManager):
    """
    Mix-in for Zope management
    """

    isPrincipiaFolderish=1
    isAnObjectManager=1
                             
    manage_options=({'label':'Edit', 'action':'manage_editForm'},
                    {'label':'Upload', 'action':'manage_uploadForm'},
                    {'label':'Properties', 'action':'manage_propertiesForm'},
                    {'label':'Security',   'action':'manage_access'},
                    )
                    
    __ac_permissions__=(
    ('Access contents information',
        ('objectIds', 'objectValues', 'objectItems', 'getfirstChild', 'getlastChild',
         'getnextSibling', 'getpreviousSibling', 'getchildNodes', 'getparentNode',
         'getownerDocument', 'getElementsByTagName', 'nextObject', 'previousObject'),
        ('Anonymous', 'Manager')),
    ('View management screens',
        ('manage', 'manage_main', 'manage_editForm', 'manage_uploadForm')),
    ('Change XML Documents',
        ('manage_edit', 'manage_upload', 'insertBefore', 'appendChild',
         'removeChild', 'normalize', 'setAttribute', 'parse')),
    ('View',
        ('', 'text_content', 'getnodeType', 'getnodeValue', 'toXML',
         'document_src', 'getAttribute', 'getattributes')),
    ('Manage properties', 
        ('manage_addProperty', 'manage_editProperties', 'manage_delProperties',
         'manage_changeProperties',)),
    )

    # Document Methods
    # ----------------
    
    manage_uploadForm=HTMLFile('documentUpload', globals())

    def get_size(self): return len(self.toXML())

    _size_changes={
        'Bigger': (5,5),
        'Smaller': (-5,-5),
        'Narrower': (0,-5),
        'Wider': (0,5),
        'Taller': (5,0),
        'Shorter': (-5,0),
        }

    def _er(self,data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST):
        dr,dc = self._size_changes[SUBMIT]
        rows=max(1,string.atoi(dtpref_rows)+dr)
        cols=max(40,string.atoi(dtpref_cols)+dc)
        e=(DateTime('GMT') + 365).rfc822()
        resp=REQUEST['RESPONSE']
        resp.setCookie('dtpref_rows',str(rows),path='/',expires=e)
        resp.setCookie('dtpref_cols',str(cols),path='/',expires=e)
        return self.manage_editForm(
            self,REQUEST,title=title,toXML=data,
            dtpref_cols=cols,dtpref_rows=rows)

    def manage_edit(self,data,title=None,SUBMIT='Change',dtpref_cols='50',
                    dtpref_rows='20',REQUEST=None):
        """
        Replaces a Documents contents with Data, Title with Title.

        The SUBMIT parameter is also used to change the size of the editing
        area on the default Document edit screen.  If the value is "Smaller",
        the rows and columns decrease by 5.  If the value is "Bigger", the
        rows and columns increase by 5.  If any other or no value is supplied,
        the data gets checked for DTML errors and is saved.
        """
        if self._size_changes.has_key(SUBMIT):
            return self._er(data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST)
        if title is not None:
            self.title=title
        try:
            self.parse(data)
        except ParseError, message:
            if REQUEST:
                return self.manage_editForm(self, REQUEST, toXML=data,
                        manage_tabs_message='<font color="red">XML Parsing Error: %s</font>' % message)
        if REQUEST:
            message="Content changed."
            return self.manage_editForm(self, REQUEST, manage_tabs_message=message)

    def manage_upload(self,file='', REQUEST=None):
        """Replace the contents of the document with the text in file."""
        try:
            self.parse(data)
        except ParseError, message:
            if REQUEST: return MessageDialog(
                    title  ='XML Parsing Error',
                    message=message,
            action ='manage_workspace') 
        if REQUEST: return MessageDialog(
                    title  ='Success!',
                    message='Your changes have been saved',
                    action ='manage_workspace')


class XMLDocument(Node.Document, Manageable, Persistent, Item, RoleManager):
    """
    Python class for XML Document nodes
    """

    meta_type='XML Document'

    manage=manage_main=manage_editForm=HTMLFile('documentEdit', globals())

    def parse(self, data):
        """
        Build a DOM tree rooted in this node given XML text or file object.
        """
        self._clear()
        p=Parser()
        p.register('Document', XMLDocument)
        p.register('Element', XMLElement)
        p.register('Text', XMLText)        
        p.register('ProcessingInstruction', XMLProcessingInstruction)
        p.reset(self)
        p.parse(data)
     
class XMLElement(Node.Element, Manageable, Persistent, Item, RoleManager):
    """
    Python class for XML Element nodes
    """

    icon="misc_/XMLDocument/xml_element.gif"

    def parse(self, data):
        """
        Build a DOM tree rooted in this node given XML text or file object.
        """
        p=Parser()
        p.register('Document', XMLDocument)
        p.register('Element', XMLElement)
        p.register('Text', XMLText)        
        p.register('ProcessingInstruction', XMLProcessingInstruction)
        p.parse(data)
        e=p.root.getdocumentElement()
        self._child_map=e._child_map
        self._child_ids=e._child_ids
        self._properties=e._properties
        for k,v in e.propertyItems():
            setattr(self, k, v)
            
    manage_editForm=HTMLFile('elementEdit', globals())

    def setAttribute(self, name, value):
        """
        Adds a new attribute. If an attribute with that name is already
        present in the element, its value is changed to be that of the
        value parameter.
        """
        try:
            self._updateProperty(name, value) 
        except:
            self._setProperty(name, value)
            
    def getAttribute(self, name):
        """
        Retrieves an attribute value by name.
        """
        return self.getProperty(name, '')

    def _clear_attributes(self):
        self._properties=()

    def getattributes(self):
        """
        A Python dictionary containing the attributes of this node
        (if it is an Element) or None otherwise.
        """
        d={}
        for p in self._properties:
            if p['type']=='string':
                d[p['id']]=getattr(self,p['id'])
        return d
        
class XMLText(Node.Text, Persistent, Item, RoleManager):
    """
    Python class for XML Text nodes.
    """
    
    
class XMLProcessingInstruction(Node.ProcessingInstruction, Persistent, Item, RoleManager):
    """
    Python class for XML Processing Instructions
    """


# Add form and method
# -------------------

addForm=HTMLFile('documentAdd', globals())

def add(self, id, title='', file='', REQUEST=None, submit=None):
    """
    Add a XML Document object with the contents of file.
    """
    ob=XMLDocument()
    ob.title=title
    ob.id=id
    if file: ob.parse(file)
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
